--
--
-- PlayerTreePlanter
--
-- # Author:  Heady
-- # date: 27.01.2019
-- # last update: 08.05.2020
--
-- > Not for commercial use  < --
-- > Not allowed for edit - Copyright (C) Heady - www.planet-ls.de < --
--


local modDir = g_currentModDirectory;

local modDescXML = loadXMLFile("modDesc", modDir .. "modDesc.xml");
local version = getXMLString(modDescXML, "modDesc.version");

print("Script: PlayerTreePlanter v"..tostring(version).." loaded by Heady");

PlayerTreePlanter = {};

local PlayerTreePlanter_mt = Class(PlayerTreePlanter, HandTool);
InitObjectClass(PlayerTreePlanter, "PlayerTreePlanter");

function PlayerTreePlanter:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = PlayerTreePlanter_mt;
	end
	--print("PlayerTreePlanter:new()")

	local self = HandTool:new(isServer, isClient, mt);
	return self;
end;


function PlayerTreePlanter:load(xmlFilename, player)
    if not PlayerTreePlanter:superClass().load(self, xmlFilename, player) then
        return false;
    end
	
	--g_treePlantManager.treeTypes[1].growthTimeHours = 1;
	
	self.activatePressedFalseTrue = false;
	self.activatePressedTrueFalse = false;
	
	self.showFieldNotOwnedWarning = false;
	self.showRestrictedZoneWarning = false;
	self.showTooManyTreesWarning = false;
	
	--[[print("self.isRegistered: "..tostring(self.isRegistered))
	if not self.isServer then
		self:register(false);
	end;
	print("self.isRegistered: "..tostring(self.isRegistered))]]
	
	self:register(false);
	
    return true;
end;


function PlayerTreePlanter:delete()

	self:unregister();
	
    PlayerTreePlanter:superClass().delete(self);
end;


function PlayerTreePlanter:onActivate(allowInput)
    PlayerTreePlanter:superClass().onActivate(self);
	--print("PlayerTreePlanter:onActivate()")
end;

function PlayerTreePlanter:onDeactivate(allowInput)
    PlayerTreePlanter:superClass().onDeactivate(self);
	--print("PlayerTreePlanter:onDeactivate()")
end;


function PlayerTreePlanter:update(dt, allowInput)

	if allowInput ~= nil then
		PlayerTreePlanter:superClass().update(self, dt, allowInput);
		
		if self.activatePressed and not self.activatePressedFalseTrue and not self.activatePressedTrueFalse then
			self.activatePressedFalseTrue = true;
		end;	
		if not self.activatePressed and self.activatePressedFalseTrue and not self.activatePressedTrueFalse then
			self.activatePressedTrueFalse = true;			
		end;
		
		if self.activatePressedFalseTrue and self.activatePressedTrueFalse then 
			local x,y,z = getWorldTranslation(g_currentMission.player.cameraNode); -- g_currentMission.player.rootNode
			local xd, yd, zd = localDirectionToWorld(g_currentMission.player.cameraNode, 0, 0, 1);
			local x1, y1, z1 = x-xd, y-yd, z-zd;
			local farmId = g_currentMission:getFarmId();
			local canPlantTree = self:canCreateTree(x1, y1, z1, farmId);
			if canPlantTree then
				self:createTree(x1, y1, z1, farmId);
				PlayerTreePlanterCreateTreeMPEvent:updateSendEvent(self, x1, y1, z1, farmId);
			end;
		
			self.activatePressedFalseTrue = false;
			self.activatePressedTrueFalse = false;	
		end;		
		
		if self.showFieldNotOwnedWarning then
			g_currentMission:showBlinkingWarning(g_i18n:getText("warning_youDontHaveAccessToThisLand"));
			self.showFieldNotOwnedWarning = false;
		end;
		if self.showRestrictedZoneWarning then
			g_currentMission:showBlinkingWarning(g_i18n:getText("warning_actionNotAllowedHere"));
			self.showRestrictedZoneWarning = false;
		end;
		if self.showTooManyTreesWarning then
			g_currentMission:showBlinkingWarning(g_i18n:getText("warning_tooManyTrees"));
			self.showTooManyTreesWarning = false;
		end;	
		
		self.activatePressed = false;
	end;
	
end;


function PlayerTreePlanter:canCreateTree(x, y, z, farmId)
	local canPlantTree = true;
    if not g_currentMission.accessHandler:canFarmAccessLand(farmId, x, z) then
		canPlantTree = false;
		self.showFieldNotOwnedWarning = true;
		--print("g_currentMission.accessHandler:canFarmAccessLand(g_currentMission:getFarmId(), x, z)")
		return false;
	end;
	if PlacementUtil.isInsideRestrictedZone(g_currentMission.restrictedZones, nil, x, y, z) then
		canPlantTree = false;
		self.showRestrictedZoneWarning = true;
		--print("PlacementUtil.isInsideRestrictedZone(g_currentMission.restrictedZones, nil, x, y, z)")
		return false;
	end;

    if not g_treePlantManager:canPlantTree() then
        self.showTooManyTreesWarning = true;
        canPlantTree = false;
		--print("g_treePlantManager:canPlantTree()")
		return false;
    end
	
	self.findObjectsCount = 0;
	local collisionMask = 4294967040;
	local numShapesSphere = overlapSphere(x, y, z, 1.5, "findObjectsCallback", self, collisionMask, true, true, false)
	if self.findObjectsCount > 0 then
		canPlantTree = false;
		return false;
	end;
	
	if g_treePlantManager.treesData ~= nil and g_treePlantManager.treesData.growingTrees ~= nil then
		for key, value in pairs(g_treePlantManager.treesData.growingTrees) do
			if value.x ~= nil and value.y ~= nil and value.z ~= nil then

				local distance = MathUtil.vector2LengthSq(x - value.x, z - value.z)
				if distance < 1.5 then
					--print("distance < 1.5")
					canPlantTree = false;
					return false;
				end;
			end
		end
	end;
	if g_treePlantManager.treesData ~= nil and g_treePlantManager.treesData.splitTrees ~= nil then
		for key, value in pairs(g_treePlantManager.treesData.splitTrees) do
			if value.x ~= nil and value.y ~= nil and value.z ~= nil then

				local distance = MathUtil.vector2LengthSq(x - value.x, z - value.z)
				if distance < 1.5 then
					--print("distance < 1.5")
					canPlantTree = false;
					return false;
				end;
			end
		end
	end;
		
	return true;	
end;

function PlayerTreePlanter:createTree(x, y, z, farmId)

	if g_server ~= nil then
		--local x,y,z = getWorldTranslation(g_currentMission.player.rootNode);
		local height = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x,y,z);
		local yRot = math.random() * 2*math.pi;
		g_treePlantManager:plantTree(1, x, height, z, 0, yRot, 0, 0);
		local stats = g_farmManager:getFarmById(farmId).stats;
		--local storeItem = g_storeManager:getItemByXMLFilename(spec.mountedSaplingPallet.configFileName);
		--local pricePerSapling = 1.5 * (storeItem.price / spec.mountedSaplingPallet:getCapacity());
		local pricePerSapling = 50;
		stats:updateStats("expenses", pricePerSapling);
		g_currentMission:addMoney(-pricePerSapling, farmId, MoneyType.PURCHASE_SEEDS);
		-- increase tree plant counter for achievements
		stats:updateStats("plantedTreeCount", 1);
		--print("tree planted")
	end;
	
end;

function PlayerTreePlanter:findObjectsCallback(transformId)
    if transformId ~= 0 and getHasClassId(transformId, ClassIds.SHAPE) then
		self.findObjectsCount = self.findObjectsCount + 1;
        --local parentNode = getParent(transformId)
		--print("findObjectsCallback(), getCollisionMask(transformId): "..tostring(getCollisionMask(transformId)).." getName(transformId): "..getName(transformId).." getName(getParent(transformId)): "..getName(getParent(transformId)))
        --if object ~= nil and getSplitType(transformId) ~= 0 then
            --self.treeCount = self.treeCount + 1
       -- end
    end
    return true
end

registerHandTool("PlayerTreePlanter", PlayerTreePlanter);


PlayerTreePlanterCreateTreeMPEvent = {};

SpeedControlSpeedMPEvent_mt = Class(PlayerTreePlanterCreateTreeMPEvent, Event);
InitEventClass(PlayerTreePlanterCreateTreeMPEvent, "PlayerTreePlanterCreateTreeMPEvent");

function PlayerTreePlanterCreateTreeMPEvent:emptyNew()
    local self = Event:new(SpeedControlSpeedMPEvent_mt);
    self.className="PlayerTreePlanterCreateTreeMPEvent";

    return self;
end;

function PlayerTreePlanterCreateTreeMPEvent:new(object, x, y, z, farmId)
    local self = PlayerTreePlanterCreateTreeMPEvent:emptyNew()
    self.object = object;
    self.x = x;
    self.y = y;
    self.z = z;
	self.farmId = farmId;

    return self;
end;

function PlayerTreePlanterCreateTreeMPEvent:readStream(streamId, connection)
    self.object = NetworkUtil.readNodeObject(streamId);	

	local x = streamReadFloat32(streamId);
	local y = streamReadFloat32(streamId);
	local z = streamReadFloat32(streamId);
	local farmId = streamReadUIntN(streamId, FarmManager.FARM_ID_SEND_NUM_BITS)

	PlayerTreePlanter:createTree(x, y, z, farmId);
	
	if not connection:getIsServer() then -- if server read then send to clients
		g_server:broadcastEvent(PlayerTreePlanterCreateTreeMPEvent:new(self.object, x, y, z, farmId), nil, connection, self.object);
	end;

end;

function PlayerTreePlanterCreateTreeMPEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.object);		
	
	streamWriteFloat32(streamId, self.x);
	streamWriteFloat32(streamId, self.y);
	streamWriteFloat32(streamId, self.z);
	streamWriteFloat32(streamId, self.farmId)
end;

function PlayerTreePlanterCreateTreeMPEvent:updateSendEvent(runSelf, x, y, z, farmId)	

	if g_server ~= nil then
		g_server:broadcastEvent(PlayerTreePlanterCreateTreeMPEvent:new(runSelf, x, y, z, farmId));
	else
		g_client:getServerConnection():sendEvent(PlayerTreePlanterCreateTreeMPEvent:new(runSelf, x, y, z, farmId));
	end;

end;